# 


"""WORDDocuments

"""

__version__='$Revision: 1.7 $'[11:-2]

from OFS.SimpleItem import SimpleItem, Item_w__name__
from OFS.PropertyManager import PropertyManager
from OFS.Cache import Cacheable
from OFS.History import Historical
from Products.ZCatalog.CatalogAwareness import CatalogAware
import Acquisition
from DateTime.DateTime import DateTime

from AccessControl import ClassSecurityInfo, getSecurityManager
from Globals import HTML, HTMLFile, DTMLFile, MessageDialog, InitializeClass

from urllib import quote
import re, string, traceback
from IWORDDocument import IWORDDocument

import os, tempfile, cgi


class WORDDocument(CatalogAware, SimpleItem, Acquisition.Implicit, 
  PropertyManager, Historical, Cacheable, Item_w__name__):

  """WORD Documents"""


  meta_type = 'WORDDocument'
  icon = 'p_/dtmldoc'

  word_src = body = raw_text = ''
  texte = ''

  index_html = None # Prevent accidental acquisition (XXX ???)

  manage_options = (
    (
      {'label':'Edit', 'action':'manage_main'},
      {'label':'View', 'action':''},
    )
    + PropertyManager.manage_options
    + Cacheable.manage_options
    + Historical.manage_options
  )

  security = ClassSecurityInfo()

  _metadata = ['title', 'author', 'publisher', 'source',
    'publication_date', 'description', 'keywords', 'summary']
  title = author = publisher = source = description = keywords = summary = ''
  _properties = (
    {'id':'title', 'type': 'string', 'mode': 'w'},
    {'id':'source', 'type': 'string', 'mode': 'w'},
  )

  __ac_permissions__=(
    ('Change WORD Documents',
     ('manage_editForm', 'manage', 'manage_main',
      'manage_edit', 'manage_upload', 'PUT',
      'manage_historyCopy',
      'manage_beforeHistoryCopy', 'manage_afterHistoryCopy',
      )
     ),
    )

  #
  # Constructor
  #
  def __init__(self, id='', title='', file='', __name__=''):
    if __name__ and not id:
      self._setId(__name__)
    else:
      self._setId(id)
    self.word_src = file
    self.title = id[:-4]
#    self._parse()

  def updateState(self):
    self._parse()
    self.ZCacheable_invalidate()
    self.reindex_object()

  manage_editForm = HTMLFile('dtml/editForm', globals())
  manage = manage_main = manage_editForm

  def manage_edit(self, data, title='', SUBMIT='Change', dtpref_cols='50',
          dtpref_rows='20', REQUEST=None):
    """
    Replaces a Documents contents with Data.

    """

    if title:
      self.title = title
    if type(data) is not type(''): 
      data = data.read()
    self.word_src = data
    self.updateState()

    if REQUEST:
      message="Content changed."
      return self.manage_main(self,REQUEST,manage_tabs_message=message)


  def manage_upload(self, file='', REQUEST=None):
    """Replace the contents of the document with the text in file."""

    if type(file) is not type(''): 
      file=file.read()
    self.word_src = file
    self.updateState()
    if REQUEST:
      message="Content uploaded."
      return self.manage_main(self,REQUEST,manage_tabs_message=message)

  def PUT(self, REQUEST, RESPONSE):
    "Handle HTTP/FTP PUT requests."

    self.dav__init(REQUEST, RESPONSE)
    self.word_src = REQUEST.get('BODY', '')
    self.updateState()
    RESPONSE.setStatus(204)
    return RESPONSE

  def manage_FTPget(self):
    "Get WORD source for FTP download"

    return self.word_src


  def document_src(self, REQUEST=None, RESPONSE=None):
    """Return unprocessed document source."""

    if RESPONSE is not None:
      RESPONSE.setHeader('Content-Type', 'application/msword')
    return self.word_src

  def __call__(self, client=None, REQUEST={}, RESPONSE=None, **kw):
    "View document"
    if RESPONSE is not None:
      RESPONSE.setHeader('Content-Type', 'application/msword')
    return self.word_src 


  def PrincipiaSearchSource(self):
    return self.texte


  def _parse(self):
    data = self.word_src
    
    self.tmpdir = tempfile.mktemp()
    os.mkdir('%s' % self.tmpdir)
    

    print self.tmpdir

    fd = open("%s/%s" % (self.tmpdir, self.id), "w")
    fd.write(data)
    fd.close()


    cmd = "cd '%s' &&  wvText '%s' '%s.txt' 2> /dev/null" \
      % (self.tmpdir, self.id, self.id[:-4])
    print cmd
    os.system(cmd)
    
    self.texte = open("%s/%s.txt" % (self.tmpdir, self.id[:-4])).read() 


    for fn in os.listdir("%s" % self.tmpdir):
      os.remove("%s/%s" % (self.tmpdir, fn))
    os.rmdir("%s" % self.tmpdir)


InitializeClass(WORDDocument)

addWORDDocumentForm = HTMLFile('dtml/addForm', globals(), 
  Kind='WORDDocument', kind='WORDDocument')

def cookId(id, file):
  if not id and hasattr(file, 'filename'):
    filename = file.filename
    #title = title or filename
    id = filename[max(
      string.rfind(filename, '/'),
      string.rfind(filename, '\\'),
      string.rfind(filename, ':'),)+1:]
  return id

def addWORDDocument(self, id, title='', file='', REQUEST=None, submit=None):
  """Add a WORDDocument object with the contents of file."""

  id = str(id)
  id = cookId(id, file)
  if type(file) is not type(''): 
    file = file.read()

  ob = WORDDocument(id=id, title=title, file=file)
  id = self._setObject(id, ob)
  if REQUEST is not None:
    try: u = self.DestinationURL()
    except: u = REQUEST['URL1']
    if submit == " Add and Edit ": u = "%s/%s" % (u, quote(id))
    REQUEST.RESPONSE.redirect(u + '/manage_main')
  return ''

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
# 
# vim:et:ts=2:ai
